﻿'use strict';

angular.module('loader', ['messages', 'storage'])

    .factory('loaderService', function ($q, $http, messagesService, localStorage) {
        var app = {
            localUrlBase: localStorage.getItem('appUrlBase') || null,
            version: localStorage.getItem('appVersion') || 0,
            url: localStorage.getItem('appUrlBlob') || null,
            devId: localStorage.getItem('devId') || null,
            indexFile: '/index.html',
            metadataUrl: 'https://www.laerciokonzen.com.br/app/softclin/metadata.json'
        };
        return {
            init: init
        };

        ////////////

        function getMetadata(url) {
            return $http.get(url, { timeout: 30000, cache: false });
        };

        function openApp() {
            // console.log(app.localUrlBase + app.indexFile);
            //alert(app.localUrlBase + app.indexFile);
            window.location = app.localUrlBase + app.indexFile;
        };

        function exitApp() {
            navigator.app.exitApp();
        };

        function needsUpdate(force, callback) {
            if (force) {
                messagesService.confirm('Existe uma atualização obrigatoria! Se cancelar, não será possivel acessar o programa.', callback, exitApp);
            } else {
                messagesService.confirm('Existe uma atualização para ser feita. Executar esta atualização agora?', callback, openApp);
            }
        };

        function contentSync(metadata) {
            var deferred = $q.defer();

            var sync = ContentSync.sync({ src: metadata.url, id: 'LK', copyCordovaAssets: true });

            sync.on('progress', function (data) {
                deferred.notify(data);
            });

            sync.on('complete', function (data) {
                deferred.resolve(data);
            });

            sync.on('error', function (e) {
                deferred.reject(e);
            });

            return deferred.promise;
        }

        function init() {

            var deferred = $q.defer();

            var url = (app.devId != null) ? app.metadataUrl + '?developerId=' + app.devId : app.metadataUrl;
            getMetadata(url).then(checkSync, error);

            function checkSync(info) {
                var appInfo = info.data;
                var force = (app.devId != null) ? true : appInfo.force;

                if (app.url == null) {
                    sync(appInfo);
                } else if (app.url != appInfo.url || app.version != appInfo.version) {
                    needsUpdate(force, function () { sync(appInfo); });
                } else {
                    openApp();
                    deferred.resolve();
                };

            };

            function error(e) {

                if (angular.isObject(e)) {
                    e.app = app;
                }

                deferred.reject(e);
                if (app.version > 0) {
                    openApp();
                }
            }

            function sync(appInfo) {
                var sync = contentSync(appInfo);

                sync.then(function (data) {
                    var urlProtocol = (device.platform != 'Win32NT') ? 'file://' : '';

                    app.localUrlBase = urlProtocol + data.localPath;
                    app.version = appInfo.version;
                    app.url = appInfo.url;

                    localStorage.setItem('appUrlBase', app.localUrlBase);
                    localStorage.setItem('appUrlBlob', app.url);
                    localStorage.setItem('appVersion', app.version);

                    openApp();
                    deferred.resolve();

                },
                    error,
                    deferred.notify);

            }

            return deferred.promise;
        }
    });
